<?php

/**
 * ----------------------------------------------------------------
 * @Copyright   : License MIT 2016-2024
 * @WebSite     : https://www.shiros.fr
 *
 * @Author      : Alexandre Caillot
 *
 * @File        : MySQLConnexionBuilder.php
 * @Created_at  : 04/02/2022
 * @Update_at   : 17/12/2024
 * ----------------------------------------------------------------
 */

namespace Luna\SQL\Connexion\Builder;

use Luna\Component\Bag\BagInterface;
use Luna\Component\Manager\TypeManager;
use Luna\Component\Manager\ValueManager;
use Luna\SQL\Connexion\Exception\SQLConnexionBuilderException;
use Luna\SQL\Connexion\MySQLConnexion;
use PDOException;

class MySQLConnexionBuilder
{
    # --------------------------------
    # Constants

    /**
     * Key : Host.
     * Correspond to the database ip address or DNS.
     */
    public const KEY_HOST = 'host';

    /**
     * Key : Port.
     * Correspond to the database service port.
     */
    public const KEY_PORT = 'port';

    /**
     * Key : Database.
     * Correspond to the database name.
     */
    public const KEY_DATABASE = 'database';

    /**
     * Key : User.
     * It's the name of the SQL account.
     */
    public const KEY_USER = 'user';

    /**
     * Key : Password.
     * It's the password of the SQL account.
     */
    public const KEY_PASSWORD = 'password';

    /**
     * Key : Options.
     * PDO options.
     */
    public const KEY_OPTIONS = 'options';

    # --------------------------------
    # Core methods

    /**
     * Build up a MySQL Connexion.
     *
     * @param string      $host
     * @param string|int  $port
     * @param string|null $database
     * @param string      $user
     * @param string      $password
     * @param array       $options
     *
     * @return MySQLConnexion
     */
    public static function build(
        string     $host,
        string|int $port,
        ?string    $database,
        string     $user,
        string     $password,
        array      $options = []
    ): MySQLConnexion {
        return new MySQLConnexion(
            host    : $host,
            port    : $port,
            database: $database,
            user    : $user,
            password: $password,
            options : $options
        );
    }

    /**
     * Build up a MySQL Connexion with data.
     *
     * Data :
     *  - Host      : Correspond to the database ip address or DNS. (<String>)
     *  - Port      : Correspond to the database service port. (<Integer>)
     *  - Name      : Correspond to the database name. (<String>)
     *  - User      : It's the name of the SQL account. (<String>)
     *  - Password  : It's the password of the SQL account. (<String>)
     *  - Options   : PDO options. (<Array> - OPTIONAL)
     *
     * @param BagInterface|array $data
     *
     * @return MySQLConnexion
     * @throws PDOException
     * @throws SQLConnexionBuilderException
     */
    public static function buildFromData(BagInterface|array $data): MySQLConnexion
    {
        // ----------------
        // Vars

        // Get data
        $data     = ValueManager::getBag(value: $data);
        $host     = $data->getAsString(key: self::KEY_HOST, default: null);
        $port     = $data->getAsInteger(key: self::KEY_PORT, default: null);
        $database = $data->getAsString(key: self::KEY_DATABASE, default: null);
        $user     = $data->getAsString(key: self::KEY_USER, default: null);
        $password = $data->getAsString(key: self::KEY_PASSWORD, default: null);
        $options  = $data->getAsArray(key: self::KEY_OPTIONS);

        // ----------------
        // Process

        // Security check : Host
        if (TypeManager::isNull($host)) {
            throw new SQLConnexionBuilderException(
                message   : "Wrong host, please check it, and be sure to have right value. You could check the description of the build process.",
                parameters: $data->all()
            );
        }

        // Security check : Port
        if (TypeManager::isNull($port)) {
            throw new SQLConnexionBuilderException(
                message   : "Wrong port, please check it, and be sure to have right value. You could check the description of the build process.",
                parameters: $data->all()
            );
        }

        // Security check : User
        if (TypeManager::isNull($user)) {
            throw new SQLConnexionBuilderException(
                message   : "Wrong user, please check it, and be sure to have right value. You could check the description of the build process.",
                parameters: $data->all()
            );
        }

        // Security check : Password
        if (TypeManager::isNull($password)) {
            throw new SQLConnexionBuilderException(
                message   : "Wrong password, please check it, and be sure to have right value. You could check the description of the build process.",
                parameters: $data->all()
            );
        }

        // Build
        return self::build(
            host    : $host,
            port    : $port,
            database: $database,
            user    : $user,
            password: $password,
            options : $options
        );
    }
}
