<?php

/**
 * ----------------------------------------------------------------
 * @Copyright   : License MIT 2016-2024
 * @WebSite     : https://www.shiros.fr
 *
 * @Author      : Alexandre Caillot
 *
 * @File        : SQLConnexionBuilder.php
 * @Created_at  : 17/12/2021
 * @Update_at   : 17/12/2024
 * ----------------------------------------------------------------
 */

namespace Luna\SQL\Connexion\Builder;

use Luna\Component\Bag\BagInterface;
use Luna\Component\Manager\ValueManager;
use Luna\SQL\Connexion\Exception\SQLConnexionBuilderException;
use Luna\SQL\Connexion\SQLConnexionInterface;
use Luna\SQL\Constant\SQLConstant;
use PDOException;

class SQLConnexionBuilder
{
    # --------------------------------
    # Constants

    /**
     * Driver type.
     *
     * Available values :
     *  - mysql
     *  - postgresql
     */
    public const KEY_TYPE = 'type';

    # --------------------------------
    # Core methods

    /**
     * Build up a SQL Connexion.
     *
     * Options :
     *  - For MySQL      : See @MySQLConnexionBuilder documentation
     *  - For PostgreSQL : See @PostgreSQLConnexionBuilder documentation
     *
     * @param string|null        $type
     * @param BagInterface|array $options
     *
     * @return SQLConnexionInterface
     * @throws SQLConnexionBuilderException
     */
    public static function build(
        ?string            $type,
        BagInterface|array $options
    ): SQLConnexionInterface {
        // ----------------
        // Process

        return match ($type) {
            SQLConstant::DRIVER_MYSQL      => MySQLConnexionBuilder::buildFromData($options),
            SQLConstant::DRIVER_POSTGRESQL => PostgreSQLConnexionBuilder::buildFromData($options),
            default                        => throw new SQLConnexionBuilderException(
                message: "Driver '{$type}' isn't supported."
            ),
        };
    }

    /**
     * Build up a SQL Connexion from data.
     *
     * Data :
     *  - Type        : Correspond to the SQL driver to use, if the driver is null will assume that it's for the MySQL
     *                  connexion. (<String>)
     *  - Others keys :
     *      - For MySQL      : See @MySQLConnexionBuilder documentation
     *      - For PostgreSQL : See @PostgreSQLConnexionBuilder documentation
     *
     * @param BagInterface|array $data
     *
     * @return SQLConnexionInterface
     * @throws PDOException
     * @throws SQLConnexionBuilderException
     */
    public static function buildFromData(BagInterface|array $data): SQLConnexionInterface
    {
        // ----------------
        // Vars

        // Get data
        $data = ValueManager::getBag($data);
        $type = $data->getAsString(key: self::KEY_TYPE, default: SQLConstant::DRIVER_MYSQL);

        // ----------------
        // Process

        // Build
        return self::build(
            type   : $type,
            options: $data
        );
    }
}
