<?php

/**
 * ----------------------------------------------------------------
 * @Copyright   : License MIT 2016-2024
 * @WebSite     : https://www.shiros.fr
 *
 * @Author      : Alexandre Caillot
 *
 * @File        : PostgreSQLConnexion.php
 * @Created_at  : 26/08/2023
 * @Update_at   : 21/12/2024
 * ----------------------------------------------------------------
 */

namespace Luna\SQL\Connexion;

use Luna\Component\Bag\BagInterface;
use Luna\Component\Manager\TypeManager;
use Luna\Component\Manager\ValueManager;
use Luna\SQL\Connexion\Exception\SQLConnexionException;
use PDO;
use PDOException;

class PostgreSQLConnexion implements SQLConnexionInterface
{
    # --------------------------------
    # Attributes

    /**
     * PostgreSQL host.
     *
     * Example:
     *  - localhost
     *  - 127.0.0.1
     */
    protected string $host;

    /**
     * PostgreSQL port.
     *
     * Example:
     *  - 5432
     *  - 1234
     */
    protected string|int $port;

    /**
     * PostgreSQL database name.
     *
     * Example:
     *  - 'my-database'
     */
    protected ?string $database;

    /**
     * PostgreSQL username.
     *
     * Example:
     *  - 'my-user'
     */
    protected string $user;

    /**
     * PostgreSQL password.
     *
     * Example:
     *  - 'my-password'
     */
    protected string $password;

    /**
     * PostgreSQL options.
     */
    protected array $options;

    /**
     * PDO connexion instance.
     */
    protected ?PDO $connexion;

    /**
     * SQL connection initialization indicator.
     */
    protected bool $isBuilt;

    # --------------------------------
    # Constructor

    /**
     * PostgreSQLConnexion constructor.
     *
     * Parameters :
     *  - Host      : Correspond to the database ip address or DNS. (<String>)
     *  - Port      : Correspond to the database service port. (<String> | <Integer>)
     *  - Database  : Correspond to the database name. (<String> | <Null>)
     *  - User      : It's the name of the SQL account. (<String>)
     *  - Password  : It's the password of the SQL account. (<String>)
     *  - Options   : PDO options. (<Array> - OPTIONAL)
     *
     * @param string             $host
     * @param string|int         $port
     * @param string|null        $database
     * @param string             $user
     * @param string             $password
     * @param BagInterface|array $options
     *
     */
    public function __construct(
        string             $host,
        string|int         $port,
        ?string            $database,
        string             $user,
        string             $password,
        BagInterface|array $options = []
    ) {
        // Set attributes
        $this->host     = $host;
        $this->port     = $port;
        $this->database = $database;
        $this->user     = $user;
        $this->password = $password;
        $this->options  = ValueManager::getArray($options);

        // Set connexion information
        $this->isBuilt = false;

        // Init connexion
        $this->init();
    }

    # --------------------------------
    # Core methods

    /**
     * @inheritDoc
     *
     * @throws PDOException
     */
    public function init(): static
    {
        // ----------------
        // Vars

        // Set DSN
        $dsn = "pgsql:host={$this->host};port={$this->port};";

        // ----------------
        // Process

        // DSN : Setup database name
        if (!TypeManager::isEmpty($this->database)) {
            $dsn .= "dbname={$this->database}";
        }

        // Close current connexion
        if ($this->isBuilt) {
            $this->close();
        }

        // Build connexion
        $this->connexion = new PDO(
            dsn     : $dsn,
            username: $this->user,
            password: $this->password,
            options : $this->options
        );
        $this->isBuilt   = true;

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function close(): static
    {
        // ----------------
        // Process

        // Close connexion
        $this->connexion = null;
        $this->isBuilt   = false;

        return $this;
    }

    # --------------------------------
    # Getters

    /**
     * @inheritDoc
     *
     * @throws SQLConnexionException
     */
    public function getConnexion(): PDO
    {
        // ----------------
        // Process

        // Check if the connexion is built
        if (!$this->isBuilt) {
            throw new SQLConnexionException(
                message: "You must init/build the connexion before get it. You have to call the 'init' method before."
            );
        }

        return $this->connexion;
    }

    # --------------------------------
    # PHP methods

    /**
     * MySQLConnexion destructor.
     */
    public function __destruct()
    {
        $this->close();
    }
}
