<?php

/**
 * ----------------------------------------------------------------
 * @Copyright   : License MIT 2016-2024
 * @WebSite     : https://www.shiros.fr
 *
 * @Author      : Alexandre Caillot
 *
 * @File        : SQLManagerResolver.php
 * @Created_at  : 17/12/2021
 * @Update_at   : 26/12/2024
 * ----------------------------------------------------------------
 */

namespace Luna\SQL\Manager\Resolver;

use Luna\Component\Container\Exception\ContainerException;
use Luna\Component\Container\LunaContainer;
use Luna\Component\Manager\ClassManager;
use Luna\Component\Manager\ValueManager;
use Luna\Config\LunaConfig;
use Luna\SQL\Connexion\Builder\SQLConnexionBuilder;
use Luna\SQL\Connexion\Exception\SQLConnexionBuilderException;
use Luna\SQL\Connexion\SQLConnexionInterface;
use Luna\SQL\Constant\SQLConstant;
use Luna\SQL\Manager\Exception\SQLManagerResolverException;
use Luna\SQL\Manager\SQLManager;
use Luna\SQL\Manager\SQLManagerInterface;

class SQLManagerResolver
{
    # --------------------------------
    # Attributes

    /**
     * Luna container.
     */
    protected LunaContainer $container;

    /**
     * Luna configuration.
     */
    protected LunaConfig $config;

    # --------------------------------
    # Constructor

    /**
     * SQLManagerResolver constructor.
     *
     * @param LunaContainer $container
     *
     * @throws ContainerException
     */
    public function __construct(
        LunaContainer $container
    ) {
        // Set attributes
        $this->container = $container;
        $this->config    = $this->container->getConfig();
    }

    # --------------------------------
    # Core methods

    /**
     * Resolve the SQL manager.
     *
     * @return SQLManagerInterface
     */
    public function resolve(): SQLManagerInterface
    {
        // ----------------
        // Process

        // Get manager from container
        $manager = $this->container->get(key: SQLConstant::CONTAINER_SQL_MANAGER);

        /**
         * Case : The container has a manager.
         * The manager is not null and implement the correct interface.
         */
        if (ClassManager::implement(interface: SQLManagerInterface::class, objectOrClass: $manager)) return $manager;

        /**
         * Default : Return a new instance of manager.
         * Instantiation is performed separately, instead of using the container getter's 'default' key,
         * to avoid a double instance in memory.
         */
        return new SQLManager();
    }

    /**
     * Resolve connexion.
     *
     * @param SQLConnexionInterface|string $connexion
     * @param SQLManagerInterface|null     $manager
     *
     * @return SQLConnexionInterface
     * @throws SQLConnexionBuilderException
     * @throws SQLManagerResolverException
     */
    public function resolveConnexion(
        SQLConnexionInterface|string $connexion,
        ?SQLManagerInterface         $manager
    ): SQLConnexionInterface {
        // ----------------
        // Vars

        // Get information
        $key       = ValueManager::getString(value: $connexion);
        $databases = $this->config->getAsBag(key: SQLConstant::CONFIG_DATABASES);

        // Conditions
        $isSQLConnexion = ClassManager::implement(interface: SQLConnexionInterface::class, objectOrClass: $connexion);
        $hasInManager   = $manager?->hasConnexion(key: $key) ?? false;
        $hasInConfig    = $databases->has(key: $key);

        // ----------------
        // Process

        // Case : This is an SQL connexion
        if ($isSQLConnexion) return $connexion;

        // Case : The connexion already exist in the manager
        if ($hasInManager) return $manager->getConnexion(key: $key);

        // Case : The connexion exists in the configuration
        if ($hasInConfig) {
            // Build connexion
            $connexion = SQLConnexionBuilder::buildFromData(
                data: $databases->getAsBag(key: $key),
            );

            // Set connexion in the manager
            $manager?->setConnexion(key: $key, connexion: $connexion);
            return $connexion;
        }

        // Default : No connexion is available
        throw new SQLManagerResolverException(
            message   : "Can't resolve the sql connexion. Please verify the key and the SQL storage config.",
            parameters: ['key' => $key, 'connexion' => $connexion],
        );
    }
}
