<?php

/**
 * ----------------------------------------------------------------
 * @Copyright   : License MIT 2016-2024
 * @WebSite     : https://www.shiros.fr
 *
 * @Author      : Alexandre Caillot
 *
 * @File        : SQLManager.php
 * @Created_at  : 14/07/2021
 * @Update_at   : 26/12/2024
 * ----------------------------------------------------------------
 */

namespace Luna\SQL\Manager;

use Luna\Component\Bag\Bag;
use Luna\Component\Bag\BagInterface;
use Luna\Component\Bag\ReadOnlyBag;
use Luna\Component\Container\Exception\ContainerException;
use Luna\Component\Container\LunaContainer;
use Luna\Component\DI\Exception\DependencyInjectorException;
use Luna\Component\Manager\ClassManager;
use Luna\Component\Manager\TypeManager;
use Luna\Component\Manager\ValueManager;
use Luna\SQL\Connexion\Exception\SQLConnexionBuilderException;
use Luna\SQL\Connexion\SQLConnexionInterface;
use Luna\SQL\Manager\Exception\SQLManagerException;
use Luna\SQL\Manager\Exception\SQLManagerResolverException;
use Luna\SQL\Manager\Resolver\SQLManagerResolver;
use Luna\SQL\Query\Exception\SQLQueryException;
use Luna\SQL\Query\SQLQuery;
use Luna\SQL\Query\SQLQueryInterface;
use ReflectionException;

class SQLManager implements SQLManagerInterface
{
    # --------------------------------
    # Attributes

    /**
     * SQL manager resolver.
     * Used to resolve connexions.
     */
    protected SQLManagerResolver $resolver;

    /**
     * SQL connexions.
     * The connections are lazy loaded.
     */
    protected BagInterface $connexions;

    # --------------------------------
    # Constructor

    /**
     * SQLManager constructor.
     *
     * @throws ContainerException
     */
    public function __construct()
    {
        // Set attributes
        $this->resolver   = new SQLManagerResolver(
            container: LunaContainer::getInstance()
        );
        $this->connexions = new Bag();
    }

    # --------------------------------
    # Core methods

    /**
     * @inheritDoc
     *
     * @param SQLConnexionInterface|string $connexion
     * @param BagInterface|array|string    $query
     * @param BagInterface|array           $variables
     * @param BagInterface|array           $options
     * @param int|null                     $status
     *
     * @return mixed
     * @throws ContainerException
     * @throws DependencyInjectorException
     * @throws ReflectionException
     * @throws SQLConnexionBuilderException
     * @throws SQLQueryException
     * @throws SQLManagerResolverException
     */
    public function execute(
        SQLConnexionInterface|string $connexion,
        BagInterface|array|string    $query,
        BagInterface|array           $variables = [],
        BagInterface|array           $options = [],
        ?int                         &$status = null
    ): mixed {
        // ----------------
        // Vars

        // Get connexion
        $connexion = $this->resolver->resolveConnexion(
            connexion: $connexion,
            manager  : $this
        );

        // Query information
        $options = ValueManager::getBag($options);

        // Get options
        $one   = $options->getAsBoolean(SQLQueryInterface::OPTION_ONE);
        $fetch = $options->getAsBoolean(SQLQueryInterface::OPTION_FETCH, true);

        // ----------------
        // Process

        // Create query
        $query = new SQLQuery($connexion, $query, $options);
        $query->setParameters($variables);

        // Execute query
        $query->execute();
        $status = $query->getStatus();

        // Get result
        if ($fetch) {
            return $query->getResult($one);
        }

        return $status;
    }

    /**
     * @inheritDoc
     *
     * @throws SQLManagerException
     */
    public function closeConnexion(string $key): void
    {
        // ----------------
        // Vars

        // Get information
        $interfaceName = SQLConnexionInterface::class;

        // Get connexion
        $connexion = $this->connexions->get($key);

        // ----------------
        // Process

        // Check if connexion exists
        if (TypeManager::isNull($connexion)) {
            throw new SQLManagerException(
                message   : 'SQL connexion not found in the manager. Please register this connexion before to use it or check config parameters.',
                parameters: ['key' => $key]
            );
        }

        // Remove the connexion
        $this->connexions->remove($key);

        // Check connexion
        if (!ClassManager::is($interfaceName, $connexion)) {
            throw new SQLManagerException(
                message   : "The SQL connexion need to implement '$interfaceName'.",
                parameters: ['key' => $key]
            );
        }

        // Close connexion
        $connexion->close();
    }

    /**
     * @inheritDoc
     *
     * @throws SQLManagerException
     */
    public function closeConnexions(): void
    {
        // ----------------
        // Process

        // Get connexions
        $connexions = $this->connexions;

        // Closes connexions
        foreach ($connexions as $key => $connexion) {
            $this->closeConnexion($key);
        }
    }

    # --------------------------------
    # Getters

    /**
     * @inheritDoc
     */
    public function hasConnexion(string $key, bool $nullable = false): bool
    {
        return $this->connexions->has($key, $nullable);
    }

    /**
     * @inheritDoc
     */
    public function getConnexion(string $key): ?SQLConnexionInterface
    {
        return $this->connexions->get($key);
    }

    /**
     * @inheritDoc
     */
    public function getConnexions(): ReadOnlyBag
    {
        return new ReadOnlyBag(
            parameters: $this->connexions
        );
    }

    # --------------------------------
    # Setters

    /**
     * @inheritDoc
     */
    public function setConnexion(string $key, SQLConnexionInterface $connexion): self
    {
        $this->connexions->set($key, $connexion);
        return $this;
    }
}
