<?php

/**
 * ----------------------------------------------------------------
 * @Copyright   : License MIT 2016-2024
 * @WebSite     : https://www.shiros.fr
 *
 * @Author      : Alexandre Caillot
 *
 * @File        : SQLManagerInterface.php
 * @Created_at  : 18/12/2021
 * @Update_at   : 20/12/2024
 * ----------------------------------------------------------------
 */

namespace Luna\SQL\Manager;

use Luna\Component\Bag\BagInterface;
use Luna\Component\Bag\ReadOnlyBag;
use Luna\SQL\Connexion\SQLConnexionInterface;
use Luna\SQL\Manager\Exception\SQLManagerResolverException;

interface SQLManagerInterface
{
    # --------------------------------
    # Core methods

    /**
     * Execute an SQL query in a specific connexion.
     *
     * Parameters :
     * - Connexion  : SQL Connexion or the connexion key
     * - Query      : SQL query
     * - Variables  : SQL query variables
     * - Options    : SQL query options
     * - Status     : Execute status
     *
     * @param SQLConnexionInterface|string $connexion
     * @param BagInterface|array|string    $query
     * @param BagInterface|array           $variables
     * @param BagInterface|array           $options
     * @param int|null                     $status
     *
     * @return mixed
     * @throws SQLManagerResolverException
     */
    public function execute(
        SQLConnexionInterface|string $connexion,
        BagInterface|array|string    $query,
        BagInterface|array           $variables = [],
        BagInterface|array           $options = [],
        ?int                         &$status = null
    ): mixed;

    /**
     * Close an existing connection of the manager.
     *
     * @param string $key
     *
     * @return void
     */
    public function closeConnexion(string $key): void;

    /**
     * Close all connections of the manager.
     *
     * @return void
     */
    public function closeConnexions(): void;

    # --------------------------------
    # Getters

    /**
     * Check is the connexion exist in the manager.
     *
     * @param string $key
     * @param bool   $nullable
     *
     * @return bool
     */
    public function hasConnexion(string $key, bool $nullable = false): bool;

    /**
     * Get an SQL connexion.
     * If the connexion doesn't exist, null is returned.
     *
     * @param string $key
     *
     * @return SQLConnexionInterface|null
     */
    public function getConnexion(string $key): ?SQLConnexionInterface;

    /**
     * Get established SQL connexions.
     */
    public function getConnexions(): ReadOnlyBag;

    # --------------------------------
    # Setters

    /**
     * Set an SQL connexion.
     *
     * @param string                $key
     * @param SQLConnexionInterface $connexion
     *
     * @return SQLManagerInterface
     */
    public function setConnexion(string $key, SQLConnexionInterface $connexion): self;
}
