<?php

/**
 * ----------------------------------------------------------------
 * @Copyright   : License MIT 2016-2024
 * @WebSite     : https://www.shiros.fr
 *
 * @Author      : Alexandre Caillot
 *
 * @File        : SQLQueryResolver.php
 * @Created_at  : 14/07/2021
 * @Update_at   : 09/11/2024
 * ----------------------------------------------------------------
 */

namespace Luna\SQL\Query\Resolver;

use Luna\Component\Bag\BagInterface;
use Luna\Component\Manager\TypeManager;
use Luna\Component\Manager\ValueManager;
use Luna\SQL\Query\SQLQueryInterface;

class SQLQueryResolver
{
    # --------------------------------
    # Core methods

    /**
     * Resolve query.
     *
     * @param BagInterface|array|string $query
     *
     * @return string
     */
    public function resolve(
        BagInterface|array|string $query
    ): string {
        // ----------------
        // Process

        return match (true) {
            TypeManager::isBag($query)   => $query->join(separator: ' '),
            TypeManager::isArray($query) => ValueManager::join(
                value    : array_map(callback: 'trim', array: $query),
                separator: ' '
            ),
            default                      => ValueManager::trim(value: $query),
        };
    }

    /**
     * Resolve query variable key.
     *
     * @param mixed $key
     *
     * @return string
     */
    public function resolveVariableKey(mixed $key): string
    {
        // ----------------
        // Vars

        // Attributes
        $key = ValueManager::getString($key);

        // ----------------
        // Process

        // Sanitize key
        $key = ValueManager::trim($key);

        // Add ':' if the key doesn't start by this character
        if (stripos($key, ':') !== 0) {
            $key = ":{$key}";
        }

        return $key;
    }

    /**
     * Resolve query variable type.
     *
     * @param int|null $type
     * @param mixed    $value
     *
     * @return int
     */
    public function resolveVariableType(?int $type, mixed $value): int
    {
        // ----------------
        // Process

        return match (true) {
            !TypeManager::isNull($type)    => $type,
            TypeManager::isNull($value)    => SQLQueryInterface::TYPE_NULL,
            TypeManager::isBoolean($value) => SQLQueryInterface::TYPE_BOOLEAN,
            TypeManager::isInteger($value) => SQLQueryInterface::TYPE_INTEGER,
            default                        => SQLQueryInterface::TYPE_STRING,
        };
    }

    /**
     * Resolve query variable value.
     *
     * @param mixed $value
     *
     * @return mixed
     */
    public function resolveVariableValue(mixed $value): mixed
    {
        // ----------------
        // Process

        return match (true) {
            TypeManager::isString($value) => ValueManager::trim($value),
            TypeManager::isObject($value) => null,
            TypeManager::isArray($value)  => null,
            default                       => $value,
        };
    }
}
