<?php

/**
 * ----------------------------------------------------------------
 * @Copyright   : License MIT 2016-2024
 * @WebSite     : https://www.shiros.fr
 *
 * @Author      : Alexandre Caillot
 *
 * @File        : SQLQuery.php
 * @Created_at  : 14/07/2021
 * @Update_at   : 19/12/2024
 * ----------------------------------------------------------------
 */

namespace Luna\SQL\Query;

use Luna\Component\Bag\Bag;
use Luna\Component\Bag\BagInterface;
use Luna\Component\Container\Exception\ContainerException;
use Luna\Component\DI\DependencyInjector;
use Luna\Component\DI\Exception\DependencyInjectorException;
use Luna\Component\Manager\TypeManager;
use Luna\Component\Manager\ValueManager;
use Luna\SQL\Connexion\SQLConnexionInterface;
use Luna\SQL\Query\Exception\SQLQueryException;
use Luna\SQL\Query\Resolver\SQLQueryResolver;
use Luna\SQL\Transaction\SQLTransaction;
use Luna\SQL\Transaction\SQLTransactionInterface;
use PDOException;
use PDOStatement;
use ReflectionException;

class SQLQuery implements SQLQueryInterface
{
    # --------------------------------
    # Attributes

    /**
     * SQL connexion.
     */
    protected SQLConnexionInterface $connexion;

    /**
     * SQL transaction.
     */
    protected SQLTransactionInterface $transaction;

    /**
     * Provided query.
     */
    protected SQLQueryResolver $resolver;

    /**
     * Provided query.
     */
    protected string $query;

    /**
     * Query parameters.
     */
    protected BagInterface $parameters;

    /**
     * PDO statement instance.
     */
    protected PDOStatement $request;

    /**
     * Query execution indicator.
     */
    protected bool $isExecuted;

    /**
     * Query status.
     */
    protected bool $status;

    /**
     * Defines the fetch mode.
     */
    protected int $fetchMode;

    /**
     * Defines the class used to format data when fetch mode is set to :
     *  - FETCH_CLASS
     *  - FETCH_OBJ
     */
    protected ?string $entity;

    # --------------------------------
    # Constructor

    /**
     * SQLQuery constructor.
     *
     * @param SQLConnexionInterface $connexion
     * @param string                $query
     * @param BagInterface|array    $options
     */
    public function __construct(
        SQLConnexionInterface $connexion,
        string                $query,
        BagInterface|array    $options
    ) {
        // Set attributes
        $this->connexion  = $connexion;
        $this->resolver   = new SQLQueryResolver();
        $this->query      = ValueManager::trim($query);
        $this->isExecuted = false;

        // Options process
        $options         = ValueManager::getBag($options);
        $this->fetchMode = $options->getAsInteger(key: self::OPTION_FETCH_MODE, default: self::FETCH_ASSOC);
        $this->entity    = $options->getAsString(key: self::OPTION_ENTITY_CLASS, default: null);

        // Query elements
        $this->transaction = new SQLTransaction(
            connexion: $this->connexion,
            options  : $options->getAsBag(key: self::OPTION_TRANSACTION)
        );
        $this->parameters  = new Bag();
    }

    # --------------------------------
    # Core methods

    /**
     * @inheritDoc
     */
    public function execute(): void
    {
        // ----------------
        // Process

        try {
            // Transaction : Begin
            $this->transaction->begin();

            // Vars
            $pdo = $this->connexion->getConnexion();

            // Prepare & Bind
            $this->request = $pdo->prepare($this->query);
            $this->bindParameters();

            // Execute
            $this->status     = $this->request->execute();
            $this->isExecuted = true;

            // Transaction : Commit
            $this->transaction->commit();
        } catch (PDOException $PDOException) {
            // Transaction : Rollback
            $this->transaction->rollback();

            // Exception
            throw $PDOException;
        }
    }

    # --------------------------------
    # Bind methods

    /**
     * Bind all parameters on the request.
     */
    protected function bindParameters(): self
    {
        // ----------------
        // Vars

        // Get information
        $request    = $this->request;
        $parameters = $this->parameters;

        // ----------------
        // Process

        // Browse parameters
        foreach ($parameters as $key => $value) {
            // Get data
            $type  = $value[self::VARIABLE_TYPE];
            $value = $value[self::VARIABLE_VALUE];

            // Bind
            $request->bindValue($key, $value, $type);
        }

        return $this;
    }

    # --------------------------------
    # Getters

    /**
     * @inheritDoc
     */
    public function getSQL(): string
    {
        return $this->query;
    }

    /**
     * @inheritDoc
     *
     * @throws SQLQueryException
     */
    public function getStatus(): bool
    {
        // ----------------
        // Process

        // Security check : The query was not executed
        if (!$this->isExecuted) {
            throw new SQLQueryException(message: 'Execute query before retrieve the result.');
        }

        return $this->status;
    }

    /**
     * @inheritDoc
     *
     * @throws ContainerException
     * @throws DependencyInjectorException
     * @throws ReflectionException
     * @throws SQLQueryException
     */
    public function getResult(bool $one = false): mixed
    {
        // ----------------
        // Process

        // Security check : The query was not executed
        if (!$this->isExecuted) {
            throw new SQLQueryException(message: 'Execute query before retrieve the result.');
        }

        // Set fetch mode
        switch ($this->fetchMode) {
            case self::FETCH_BOTH:
            case self::FETCH_BOUND:
            case self::FETCH_LAZY:
            case self::FETCH_NAMED:
            case self::FETCH_NUM:
            case self::FETCH_OBJ:
                $this->request->setFetchMode($this->fetchMode);
                break;

            case self::FETCH_CLASS:
            case self::FETCH_INTO:
                $class = DependencyInjector::callConstructor($this->entity);
                $this->request->setFetchMode($this->fetchMode, $class);
                break;

            default:
                $this->request->setFetchMode(self::FETCH_ASSOC);
                break;
        }

        // Get result(s)
        return $one ? $this->request->fetch() : $this->request->fetchAll();
    }

    # --------------------------------
    # Setters

    /**
     * @inheritDoc
     */
    public function setParameter(
        string $key,
        mixed  $value,
        ?int   $type = self::TYPE_STRING
    ): self {
        // ----------------
        // Process

        // Resolve data
        $key   = $this->resolver->resolveVariableKey($key);
        $type  = $this->resolver->resolveVariableType($type, $value);
        $value = $this->resolver->resolveVariableValue($value);

        // Set parameter
        $this->parameters->set(
            key  : $key,
            value: [
                self::VARIABLE_TYPE  => $type,
                self::VARIABLE_VALUE => $value
            ]
        );

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function setParameters(BagInterface|array $parameters): self
    {
        // ----------------
        // Process

        // Browse provided parameters
        foreach ($parameters as $key => $value) {
            // Generate parameter data
            $type  = match (true) {
                TypeManager::isArray($value) => $value[self::VARIABLE_TYPE] ?? null,
                default                      => null
            };
            $value = match (true) {
                TypeManager::isArray($value) => $value[self::VARIABLE_VALUE] ?? null,
                default                      => $value
            };

            // Set parameter
            $this->setParameter(
                key  : $key,
                value: $value,
                type : $type
            );
        }

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function setFetchMode(int $mode): self
    {
        $this->fetchMode = $mode;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function setEntity(string $entity): self
    {
        $this->entity = $entity;
        return $this;
    }
}
