<?php

/**
 * ----------------------------------------------------------------
 * @Copyright   : License MIT 2016-2024
 * @WebSite     : https://www.shiros.fr
 *
 * @Author      : Alexandre Caillot
 *
 * @File        : SQLQuery.php
 * @Created_at  : 18/12/2021
 * @Update_at   : 09/11/2024
 * ----------------------------------------------------------------
 */

namespace Luna\SQL\Query;

use PDO;

interface SQLQueryInterface
{
    # --------------------------------
    # Constants

    /**
     * Defines value's type as BOOLEAN.
     */
    public const TYPE_BOOLEAN = PDO::PARAM_BOOL;

    /**
     * Defines value's type as STRING.
     */
    public const TYPE_STRING = PDO::PARAM_STR;

    /**
     * Defines value's type as INTEGER.
     */
    public const TYPE_INTEGER = PDO::PARAM_INT;

    /**
     * Defines value's type as NULLABLE.
     */
    public const TYPE_NULL = PDO::PARAM_NULL;

    /**
     * @Boolean
     * Defines if the query returns one or multiple elements.
     */
    public const OPTION_ONE = 'one';

    /**
     * @String
     * Defines the class used to format data when fetch mode is set to :
     *  - FETCH_CLASS
     *  - FETCH_OBJ
     */
    public const OPTION_ENTITY_CLASS = 'entity_class';

    /**
     * @Boolean
     * Defines if the query returns data or not.
     */
    public const OPTION_FETCH = 'fetch';

    /**
     * @Integer
     * Defines the fetch mode.
     */
    public const OPTION_FETCH_MODE = 'fetch_mode';

    /**
     * Allows to configure the transaction.
     *
     * @see SQLTransactionInterface for more information
     */
    public const OPTION_TRANSACTION = 'transaction';

    /**
     * Defines the variable type.
     *
     * @see SQLQueryInterface::TYPE_BOOLEAN
     * @see SQLQueryInterface::TYPE_STRING
     * @see SQLQueryInterface::TYPE_INTEGER
     * @see SQLQueryInterface::TYPE_NULL
     *
     * Example :
     *  - string
     *  - integer
     *  - boolean
     *  - null
     */
    public const VARIABLE_TYPE = 'type';

    /**
     * Defines the variable value.
     */
    public const VARIABLE_VALUE = 'value';

    /**
     * Defines the fetch as ASSOC.
     * To see more about PDO fetch constants, you can follow the link below.
     *
     * @link https://php.net/manual/en/pdo.constants.php#pdo.constants.fetch-assoc
     */
    public const FETCH_ASSOC = PDO::FETCH_ASSOC;

    /**
     * Defines the fetch as BOTH.
     * To see more about PDO fetch constants, you can follow the link below.
     *
     * @link https://php.net/manual/en/pdo.constants.php#pdo.constants.fetch-both
     */
    public const FETCH_BOTH = PDO::FETCH_BOTH;

    /**
     * Defines the fetch as BOUND.
     * To see more about PDO fetch constants, you can follow the link below.
     *
     * @link https://php.net/manual/en/pdo.constants.php#pdo.constants.fetch-bound
     */
    public const FETCH_BOUND = PDO::FETCH_BOUND;

    /**
     * Defines the fetch as CLASS.
     * To see more about PDO fetch constants, you can follow the link below.
     *
     * @link https://php.net/manual/en/pdo.constants.php#pdo.constants.fetch-class
     */
    public const FETCH_CLASS = PDO::FETCH_CLASS;

    /**
     * Defines the fetch as INTO.
     * To see more about PDO fetch constants, you can follow the link below.
     *
     * @link https://php.net/manual/en/pdo.constants.php#pdo.constants.fetch-into
     */
    public const FETCH_INTO = PDO::FETCH_INTO;

    /**
     * Defines the fetch as LAZY.
     * To see more about PDO fetch constants, you can follow the link below.
     *
     * @link https://php.net/manual/en/pdo.constants.php#pdo.constants.fetch-lazy
     */
    public const FETCH_LAZY = PDO::FETCH_LAZY;

    /**
     * Defines the fetch as NAMED.
     * To see more about PDO fetch constants, you can follow the link below.
     *
     * @link https://php.net/manual/en/pdo.constants.php#pdo.constants.fetch-named
     */
    public const FETCH_NAMED = PDO::FETCH_NAMED;

    /**
     * Defines the fetch as NUM.
     * To see more about PDO fetch constants, you can follow the link below.
     *
     * @link https://php.net/manual/en/pdo.constants.php#pdo.constants.fetch-num
     */
    public const FETCH_NUM = PDO::FETCH_NUM;

    /**
     * Defines the fetch as OBJ.
     * To see more about PDO fetch constants, you can follow the link below.
     *
     * @link https://php.net/manual/en/pdo.constants.php#pdo.constants.fetch-obj
     */
    public const FETCH_OBJ = PDO::FETCH_OBJ;

    # --------------------------------
    # Core methods

    /**
     * Execute the query.
     * If an error is occurred, a rollback is performed.
     *
     * @return void
     */
    public function execute(): void;

    # --------------------------------
    # Getters

    /**
     * Get the query.
     *
     * @return string
     */
    public function getSQL(): string;

    /**
     * Retrieve the request status.
     *
     * @return bool
     */
    public function getStatus(): bool;

    /**
     * Retrieve the request results.
     *
     * @param bool $one
     *
     * @return mixed
     */
    public function getResult(bool $one = false): mixed;

    # --------------------------------
    # Setters

    /**
     * Allow to set a parameter in the query string.
     *
     * Example :
     *  - Query : SELECT * FROM TestTable TT WHERE TT.id = :id
     *  - After : $query->setParameter('id', 0);
     *
     * @param string   $key
     * @param mixed    $value
     * @param int|null $type
     *
     * @return SQLQueryInterface
     */
    public function setParameter(
        string $key,
        mixed  $value,
        ?int   $type = self::TYPE_STRING
    ): self;

    /**
     * That the same thing of 'setParameter' method, but for an array of values.
     *
     * @param array $parameters
     *
     * @return SQLQueryInterface
     */
    public function setParameters(array $parameters): self;

    /**
     * Permit to change the PDO fetch mode.
     *
     * @param int $mode
     *
     * @return SQLQueryInterface
     */
    public function setFetchMode(int $mode): self;

    /**
     * Permit to change the PDO entity fetch.
     *
     * @param string $entity
     *
     * @return SQLQueryInterface
     */
    public function setEntity(string $entity): self;
}
