<?php

/**
 * ----------------------------------------------------------------
 * @Copyright   : License MIT 2016-2024
 * @WebSite     : https://www.shiros.fr
 *
 * @Author      : Alexandre Caillot
 *
 * @File        : SQLTransaction.php
 * @Created_at  : 14/07/2021
 * @Update_at   : 09/11/2024
 * ----------------------------------------------------------------
 */

namespace Luna\SQL\Transaction;

use Luna\Component\Bag\BagInterface;
use Luna\Component\Manager\ValueManager;
use Luna\SQL\Connexion\SQLConnexionInterface;
use PDO;

class SQLTransaction implements SQLTransactionInterface
{
    # --------------------------------
    # Attributes

    /**
     * SQL connexion object.
     */
    protected SQLConnexionInterface $connexion;

    /**
     * PDO instance.
     */
    protected PDO $pdoConnexion;

    /**
     * Defines if the transaction system is enabled.
     */
    protected bool $enable;

    # --------------------------------
    # Constructor

    /**
     * SQLTransaction constructor.
     *
     * @param SQLConnexionInterface $connexion
     * @param BagInterface|array    $options
     */
    public function __construct(
        SQLConnexionInterface $connexion,
        BagInterface|array    $options = []
    ) {
        // Set attributes
        $this->connexion    = $connexion;
        $this->pdoConnexion = $this->connexion->getConnexion();

        // Options
        $options      = ValueManager::getBag($options);
        $this->enable = $options->getAsBoolean(self::OPTION_ENABLE);
    }

    # --------------------------------
    # Core methods

    /**
     * @inheritDoc
     */
    public function begin(): void
    {
        // ----------------
        // Process

        // Security check : Not enable
        if (!$this->enable) return;

        // Start transaction
        $this->pdoConnexion->beginTransaction();
    }

    /**
     * @inheritDoc
     */
    public function rollback(): void
    {
        // ----------------
        // Process

        // Security check : Not enable
        if (!$this->enable) return;

        // Rollback transaction
        $this->pdoConnexion->rollBack();
    }

    /**
     * @inheritDoc
     */
    public function hasIn(): bool
    {
        // ----------------
        // Process

        // Security check : Not enable
        if (!$this->enable) return false;

        // Verify if is in a transaction
        return $this->pdoConnexion->inTransaction();
    }

    /**
     * @inheritDoc
     */
    public function commit(): void
    {
        // ----------------
        // Process

        // Security check : Not enable
        if (!$this->enable) return;

        // Commit transaction
        $this->pdoConnexion->commit();
    }
}
